/******************************************************************************

Project:
    DirectNet MPF

File:
    devDnMpf.h

Description:
    Header desribing common device support routines for directNet over MPF

Author:
    Andrew Johnson
Created:
    September 2001
Version:
    devDnMpf.h,v 1.1 2002/01/07 22:36:18 anj Exp

******************************************************************************/

#ifndef INC_devDnMpf_H
#define INC_devDnMpf_H

/* Required include files */
#include <dbCommon.h>
#include <devSup.h>
#include <link.h>


#define PLCWORDBITS	16	/* Bits per Word */
#define PLCWORDMASK	0xffff	/* All word bits set */
#define PLCWORDBYTES	2	/* Bytes per Word */
#define DNREFOFFSET	1	/* PLC Address offset over DirectNet */


/* limits */
#define MAXDNSLAVEID 	90	/* DirectNet limitation */

#define WRITEMINADDR	02000	/* Don't write below V02000 */
#define WRITEMAXADDR	02777	/* Don't write above V02777 */


typedef struct {
    long	number;
    DEVSUPFUN	report;
    DEVSUPFUN	init;
    DEVSUPFUN	init_record;
    DEVSUPFUN	get_ioint_info;
    DEVSUPFUN	read_write;
    DEVSUPFUN	special_linconv;
} XXDSET;


extern int devDnMpfDebug;


/* Device information structures */

struct plcInfo {
    struct plcInfo *pNext;
    const char* name;
    const char* server;
    int location;
    unsigned short slaveId;
    unsigned short alarm;
    struct rdCache *rdCache;
    struct wrCache *wrCache;
    unsigned long nRdReqs;
    unsigned long nWrReqs;
    unsigned long nSuccess;
    unsigned long nDnFail;
    unsigned long nMpFail;
};

struct plcAddr {
    struct plcInfo *plcInfo;
    unsigned short vAddr;
    unsigned char bitNum;
};

typedef void (*dnPlcReportFn)(int detail, struct plcInfo *pPlc);


/* Common routines in devDnMpf.c */

int createDnMpfPLC(const char* pname, int slaveId, const char* pserver, int location);
struct plcInfo *dnMpfPlc(const char* pname);
int dnMpfAddr(struct dbCommon *prec, struct plcAddr *paddr, struct link *plink);
void dnMpfReport(int detail, dnPlcReportFn ioReport);

#endif /* INC_devDnMpf_H */
