/******************************************************************************

Project:
    DirectNet MPF

File:
    directNetClient.cc

Description:
    Client support routines for directNet over MPF

Author:
    Andrew Johnson
Created:
    August 2001
Version:
    directNetClient.cc,v 1.1 2002/01/07 22:36:19 anj Exp

******************************************************************************/

#include <vxWorks.h>
#include <stdio.h>
#include <string.h>

#include "Char8ArrayMessage.h"
#include "ConnectMessage.h"

#include "directNetMpf.h"
#include "directNetClient.h"


int dnClientDebug = 0;

struct dnMpfClient {
public: // Member functions
    dnMpfClient(plcMessage* pPlcMsg);
    int connect(const char *pserver, int location);
    int send(struct plcMessage *pPlcMsg);
    
private:
    static void MessageClientCallback(Message *pMpfMsg, void *pPvt);
    void doCallback(Message *pMpfMsg, plcMessage* pPlcMsg);
    
private: // Data
    MessageClient *pMsgClient;
};


// Member Functions

inline dnMpfClient::dnMpfClient(plcMessage* pPlcMsg) :
    pMsgClient(new MessageClient(MessageClientCallback, pPlcMsg))
    {}

inline int dnMpfClient::connect(const char *pserver, int location) {
    return pMsgClient->bind(const_cast<char*>(pserver), location);
}

int dnMpfClient::send(struct plcMessage *pPlcMsg) {
    Char8ArrayMessage *pMpfMsg = new Char8ArrayMessage;
    
    // pMpfMsg->setClientType(static_cast<int>(pPlcMsg));
    
    pMpfMsg->cmd     = pPlcMsg->cmd;
    pMpfMsg->address = pPlcMsg->addr;
    pMpfMsg->extra   = pPlcMsg->len;
    
    if (pPlcMsg->cmd & WRITECMD) {
	pMpfMsg->allocValue(pPlcMsg->len);
	memcpy(pMpfMsg->value, pPlcMsg->pdata, pPlcMsg->len);
	pMpfMsg->setSize(pPlcMsg->len);
    }
    
    if (dnClientDebug)
	printf("dnClient: Sending write cmd=%#x, addr=%#o, extra=%d, size=%d\n",
		pMpfMsg->cmd, pMpfMsg->address, pMpfMsg->extra, pMpfMsg->getSize());
    
    int status = pMsgClient->send(pMpfMsg);
    if (status) delete pMpfMsg;
    
    return status;
}

void dnMpfClient::doCallback(Message *pMpfMsg, plcMessage* pPlcMsg) {
    switch (pMpfMsg->getType()) {
    ConnectMessage *pConnMsg;
    Char8ArrayMessage *pCharMsg;
    
    case messageTypeConnect:
	pConnMsg = static_cast<ConnectMessage*>(pMpfMsg);
	if (pPlcMsg->connStat)
	    (*pPlcMsg->connStat)(pPlcMsg, (pConnMsg->status != connectYes));
	break;
	
    case messageTypeChar8Array:
	pCharMsg = static_cast<Char8ArrayMessage*>(pMpfMsg);
	pPlcMsg->status = pCharMsg->status;
	if (((pPlcMsg->cmd & WRITECMD) == 0) &&
	    (pPlcMsg->status == DN_SUCCESS)) {
	    memcpy(pPlcMsg->pdata, pCharMsg->value, pPlcMsg->len);
	}
	(*pPlcMsg->callback)(pPlcMsg);
	break;
	
    default:
	break;
    }
    delete pMpfMsg;
}

void dnMpfClient::MessageClientCallback(Message *pMpfMsg, void *pPvt) {
    plcMessage *pPlcMsg = static_cast<plcMessage*>(pPvt);
    pPlcMsg->pClient->doCallback(pMpfMsg, pPlcMsg);
}


// C wrappers

extern "C" {

int initDnMpfClient(struct plcMessage* pPlcMsg) {
    pPlcMsg->pClient = new dnMpfClient(pPlcMsg);
    // FIXME: exception handling for OOM error?
    return pPlcMsg->pClient->connect(pPlcMsg->server, pPlcMsg->location);
}

int dnMpfClientSend(struct plcMessage *pPlcMsg) {
    pPlcMsg->status = 0;
    return pPlcMsg->pClient->send(pPlcMsg);
}

} // extern "C"

